/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.farm.model;

import java.io.IOException;
import java.util.List;
import nl.cloudfarming.client.crop.model.CropDefinition;
import nl.cloudfarming.client.farm.model.FieldPlan;
import nl.cloudfarming.client.farm.model.CropField;
import nl.cloudfarming.client.farm.model.CropProductionUnit;
import org.openide.filesystems.FileObject;

/**
 * Manager for fieldplan objects.
 * 
 * Implementations should be stateless and able to be used by multiple
 * farm projects at the same time.
 * 
 * All methods have a context argument like a FileObject. Method implementations
 * should not act beyond the boundries of that context.
 * 
 * 
 * @author Timon Veenstra <monezz@gmail.com>
 */
public interface FieldPlanManager {

    /**
     * loads a fieldplan from a fileobject
     *
     * @param fileObject
     * @return
     * @throws DataStorageException when fieldplan could not be loaded
     */
    FieldPlan loadFieldPlan(FileObject fileObject) throws IOException;

    /**
     * save a field plan
     * 
     * @param fieldPlan
     * @param fileObject
     * @throws DataStorageException when field plan could not be saved
     */
    void save(FieldPlan fieldPlan, FileObject fileObject) throws IOException;


    
    /**
     * loads a crop production unit from a fileobject
     *
     * @param fileObject
     * @return
     * @throws DataStorageException crop field could not be loaded
     */
    CropProductionUnit loadCropProductionUnit(FileObject cropProductionUnitObject) throws IOException;

    /**
     * save a crop field
     * 
     * @param cropProductionUnit
     * @param fileObject
     * @throws DataStorageException crop field could not be saved
     */
    void save(CropProductionUnit cropProductionUnit, FileObject productionUnitFolder) throws IOException;    
    
    /**
     * Adds a new fieldPlan to the provided fieldPlanFolder
     * @param harvestYear the harvest year of the new field plan
     * @return created folder containing the fieldplan
     */
    FileObject addFieldPlan(String name, Integer harvestYear);
    
    /**
     * Adds a new FieldPlan and ads the provided CropFields to it while creating it
     * @param name
     * @param harvestYear
     * @param cropFields
     * @return created folder containing the fieldplan
     */
    FileObject addFieldPlan(String name, Integer harvestYear, List<CropField> cropFields);  
    
    
    /**
     * get the folder containing  the fieldplan folders
     * 
     * @return 
     */
    FileObject getFieldPlansFolder();
    
    /**
     * Add a new production unit to a fieldplan.
     * 
     * @param name of the production unit
     * @param crop 
     * @param fieldPlanFolder
     * @return 
     */
    FileObject addCropProductionUnit(String name, CropDefinition crop, FileObject fieldPlanFolder);
    
    
    
    
}
